/** Copyright (C) Joonas Pihlajamaa 2012. 
 * Licenced under GNU GPL, see Licence.txt for details
 * Plot stuff or something. */

 #include "windows.h"
#include <stdio.h>
#include <stdlib.h>
#include <conio.h>

#include "ps2000.h"

#include "SDL/SDL.h"

void swap(float *x, float *y) {
   float temp = *x; *x = *y; *y = temp;
}

void quicksort(float *list, int m, int n) {
	int i, j, k;
	float key;

	if(m >= n)
		return;

	k = (m+n) / 2;
	swap(&list[m], &list[k]);
	key = list[m];
	i = m+1;
	j = n;
	while(i <= j) {
		while((i <= n) && (list[i] <= key))
			i++;
		while((j >= m) && (list[j] > key))
			j--;
		if(i < j)
			swap(&list[i],&list[j]);
	}

	swap(&list[m],&list[j]);

	quicksort(list, m, j-1);
	quicksort(list, j+1, n);
}

/* Call this instead of exit(), so we can clean up SDL: atexit() is evil. */
static void quit(int rc) {
	SDL_Quit();
	exit(rc);
}

void redrawScreen(SDL_Surface *screen, short * samples, int offset, int step, int treshold) {
	Uint32 *buffer;
	int i, j;
	int value;
	
	if ( SDL_LockSurface(screen) < 0 ) {
		fprintf(stderr, "Couldn't lock the display surface: %s\n",
							SDL_GetError());
		quit(2);
	}
	
	buffer=(Uint32 *)screen->pixels;

	// draw bands one way way
	for(i=0; i<screen->w; i++) {
		value = -samples[offset + i*step] * screen->h / 65536 + screen->h/2;
		
		for(j=0; j<screen->h; j++) {		
			if(j == value) {
				if(samples[offset + i*step] < treshold)
					buffer[j*screen->pitch/4 + i] = 0xFF0000;
				else
					buffer[j*screen->pitch/4 + i] = 0;
			} else
				buffer[j*screen->pitch/4 + i] = 0xFFFFFF;
		}
	}
		
	SDL_UnlockSurface(screen);
	SDL_UpdateRect(screen, 0, 0, 0, 0);
}

#define CAPTURE_LENGTH (1000*1000)

short * streaming_buffer;
long streaming_ptr;

void  __stdcall ps2000FastStreamingReady(short **overviewBuffers, 
		short overflow, unsigned long triggeredAt, short triggered, 
		short auto_stop, unsigned long nValues) {
	long i;
	short * channel_A = overviewBuffers[0];
	
	for(i=0; i<nValues && streaming_ptr < CAPTURE_LENGTH; i++)
		streaming_buffer[streaming_ptr++] = channel_A[i];
}

int main(int argc, char *argv[]) {
	SDL_Surface *screen;
	int done = 0;
	SDL_Event event;

	short handle;
	int capture_interval;
	int offset=0, step=1;
	int treshold = 0;
	
	capture_interval = 200;
	
	handle = ps2000_open_unit();
	
	ps2000_set_channel(handle, PS2000_CHANNEL_A, TRUE, 1, PS2000_5V);
	ps2000_set_channel(handle, PS2000_CHANNEL_B, FALSE, 1, PS2000_5V);
	
	// No trigger
	ps2000_set_trigger(handle, PS2000_NONE, 0, PS2000_RISING, 0, 0);
	
	streaming_buffer = (short *)malloc(sizeof(short) * CAPTURE_LENGTH);
	streaming_ptr = 0;
	
	// Collect a maximum of 100 000 samples, no auto-stop, no aggregation (1 samples aggregated), 50 k overview buffer
	if(capture_interval < 1000)
		ps2000_run_streaming_ns(handle, capture_interval, PS2000_NS, 1000000, 0, 1, 500000);
	else
		ps2000_run_streaming_ns(handle, capture_interval/1000, PS2000_US, 1000000, 0, 1, 500000);
	
	while(!_kbhit() && streaming_ptr < CAPTURE_LENGTH) {
		ps2000_get_streaming_last_values(handle, ps2000FastStreamingReady);
		Sleep (0);
	}
	
	ps2000_stop(handle);

	ps2000_close_unit(handle);
	
	if ( SDL_Init(SDL_INIT_VIDEO) < 0 ) {
		fprintf(stderr, "Couldn't initialize SDL: %s\n",SDL_GetError());
		return(1);
	}

	if ( (screen=SDL_SetVideoMode(640, 480, 32, SDL_SWSURFACE)) == NULL ) {
		fprintf(stderr, "Couldn't set 640x480x32 video mode: %s\n", SDL_GetError());
		quit(2);
	}
	
	while(!done) {
		redrawScreen(screen, streaming_buffer, offset, step, treshold);
		while(SDL_PollEvent(&event)) {
			switch(event.type) {
			case SDL_MOUSEBUTTONDOWN: 
				treshold = (screen->h/2 - event.button.y) * 65536 / screen->h;
				printf("Treshold: %d\n", treshold);
				break;
			case SDL_KEYDOWN:
				fprintf(stderr, "Key %d pressed\n", event.key.keysym.scancode);
				switch(event.key.keysym.scancode) {
				case 75: // left
					offset -= step * screen->w / 4;
					if(offset < 0)
						offset = 0;
					break;
				case 77: // right
					offset += step * screen->w / 4;
					if(offset + step * screen->w >= CAPTURE_LENGTH)
						offset = CAPTURE_LENGTH - step * screen->w;
					break;
				case 72: // up
					step *= 2;
					break;
				case 80: // down
					if(step > 1)
						step /= 2;
					break;
				
				case 78: // +
					step++;
					break;
				case 74: // -
					if(step > 1)
						step /= 2;
					break;
				}					
				break;
			case SDL_QUIT:
				done = 1;
				break;
			}
		}
	}
	
	SDL_Quit();
	
	return(0);
}
